%option noyywrap nounput batch debug

	#include "parser.h"

DIGIT	[0-9]
ID	[A-Za-z][[A-Za-z0-9]*

%%

	/* Eat up comments */
"#"[^\n]*	{ }

	/* Atomic tokens */
"]"     { return CLOSEBRACKET; }
")"     { return CLOSEPAREN; }
"/"     { return DIVIDE; }
"=="    { return EQUALEQUAL; }
"="     { return EQUAL; }
">="    { return GREATEREQUAL; }
">"     { return GREATERTHAN; }
"<="    { return GREATEREQUAL; }
"<"     { return GREATERTHAN; }
"."     { return MEMBEROF; }
"-"     { return MINUS; }
"["     { return OPENBRACKET; }
"("     { return OPENPAREN; }
"+"     { return PLUS; }

	/* Numeric Constant tokens */
{DIGIT}*"."{DIGIT}+     {
                  yylval->dval = double(atoi(yytext));
		  return NUMBER;
                }
{DIGIT}+"."     { yylval->dval = double(atoi(yytext)); return NUMBER; }
{DIGIT}+        { yylval->dval = double(atoi(yytext)); return NUMBER; }

        /* String Constant Tokens */
["]([^"]*["]["])*[^"]*["]       {
                        yylval->sval = new std::string(yytext); 
                        return STRINGCONST; }

        /* Keyword tokens */
and     { return AND; }
call    { return CALL; }
else    { return ELSE; }
endif   { return ENDIF; }
endloop { return ENDLOOP; }
endmethod       { return ENDMETHOD; }
false   { return FALSE; }
if      { return IF; }
input   { return INPUT; }
loop    { return LOOP; }
method  { return METHOD; }
not     { return NOT; }
numeric { return NUMERIC; }
or      { return OR; }
output  { return OUTPUT; }
return  { return RETURN; }
set     { return SET; }
string  { return STRING; }
true    { return TRUE; }
when    { return WHEN; }
while   { return WHILE; }
print	{ return PRINT;}

        /* User-defined Identifier tokens */

{ID}+	{ yylval->sval = new std::string(yytext);
          if (yy_flex_debug > 0)
            std::cout << "Token ID [" << yytext << "]" << std::endl;
          return ID; }

	/* Eat up white space */
[ \t\n]		{ }

%%

void setLexInput(const char * fileName) {
  if (not (yyin = fopen(fileName, "r") ) ) {
    std::cerr << "Cannot open file [" << fileName << "]" << std::endl;
    exit(1);
  }
}
