/* Using GLR grammar */
     %glr-parser
     %expect-rr 1


%skeleton "lalr1.cc"
%define "parser_class_name" "parser"
%defines

%{
 #include "ICode.h"
 
 #include <iostream>
 #include <string>
 #include <map>
 extern std::map<std::string, double> variables;

 #include "ICode.h"
%}

%union 
{
    double dval;
    // const char * sval;
    const std::string * sval;
};


%{
 // Announce to Flex the prototype we want for lexing function, ...
 # define YY_DECL						 \
 int yylex(YYSTYPE* yylval)
 // ... and declare it for the parser's sake.
 YY_DECL;
%}

%token AND CALL CONDITION ELSE ENDIF ENDLOOP ENDMETHOD FALSE IF 
%token INPUT LOOP METHOD NOT NUMERIC OR OUTPUT RETURN SET STRING
%token TRUE WHEN WHILE

%token CLOSEBRACKET CLOSEPAREN DIVIDE EQUAL EQUALEQUAL GREATEREQUAL GREATERTHAN
%token LESSEQUAL LESSTHAN MEMBEROF MINUS MULT OPENBRACKET OPENPAREN PLUS CONDITIONCONST

%token  <dval> NUMBER
%token  <sval> ID
%token  <sval> STRINGCONST

%token         SET 
%token         PRINT

%left EQUAL
%left '+' '-'
%left '*' '/'
%left '^'
%left OPENPAREN CLOSEPAREN

%%


pgm: stmt_or_method_List	
	;

/* missing method declaration */
stmt_or_method_List : stmtList 
;

stmtList: stmtList stmt 
   | /* empty */   
   ;

stmt: 		callStmt  
			| ifStmt  
			| loopStmt  
			| setStmt			
			| printStmt  
		;

callStmt : CALL methodCall   
		;

ifStmt : IF expr stmtList ENDIF  
		| IF  expr stmtList ELSE stmtList ENDIF  
		;

loopStmt : loopHead stmtList loopTail   
		;

loopHead : LOOP								
                | LOOP OPENPAREN setStmt CLOSEPAREN		
                | LOOP WHILE expr			
                | LOOP OPENPAREN setStmt CLOSEPAREN WHILE expr	
                ;

loopTail : ENDLOOP							
                | ENDLOOP WHEN expr			
			;


methodDeclaration : methodSignature stmtList methodTail
			;
			
methodSignature : METHOD ID OPENPAREN CLOSEPAREN
				| METHOD ID OPENPAREN idList CLOSEPAREN
				;
				
methodTail : ENDMETHOD
			| ENDMETHOD RETURN expr
			;
			
idList : ID
		| idList "," ID
		;							
		
exprList : expr				
		| expr "," exprList	
		;

expr:    OPENPAREN expr CLOSEPAREN {$<dval>$ = $<dval>2;}
		| expr '+' expr        {$<dval>$ = $<dval>1 + $<dval>3;    }
        | expr '-' expr       {$<dval>$ = $<dval>1 - $<dval>3;    }
        | expr '*' expr       {$<dval>$ = $<dval>1 * $<dval>3;    }
        | expr '/' expr       {$<dval>$ = $<dval>1 / $<dval>3;    }
        | '-' expr				{$<dval>$ = -$<dval>2;        }
		| operand             
        ;


methodCall : ID OPENPAREN exprList CLOSEPAREN   
			| ID OPENPAREN CLOSEPAREN		
		;
     
operand : NUMBER						
			| STRINGCONST				
			| CONDITIONCONST			
              | methodCall				
              | ID "." methodCall		
              | ID OPENBRACKET expr CLOSEBRACKET			
              | ID						
		;

setStmt: SET ID EQUAL expr {
      std::cout << "Set " << (*$2) << " = " << ($<dval>4) << std::endl; 
      //variables[*$2] = $<dval>4;
   };

printStmt: PRINT ID {std::cout << "print id [" << (*$2) << "]"
              << " = " << variables[*$2] << std::endl; };
%%

namespace yy {

   void parser::error(const yy::location & lloc, const std::string & m) {
      std::cerr << m << std::endl;
   }

}

void setLexInput(const char * fileName);
extern int yy_flex_debug;

std::map<std::string, double> variables;


#include <fstream>

int addTuple(int opCode, Operand &op1, Operand &op2, Operand &op3){
	
	
	return 0;
}


int main(int argc, char * argv[]) {
  yy::parser myParser;
  yy_flex_debug = 0;
  int fileNameIndex = 1;

  myParser.set_debug_level(false);
  if (argc > fileNameIndex and argv[fileNameIndex] == std::string("-t")) {
    myParser.set_debug_level(true);
    yy_flex_debug = 1;
    fileNameIndex++;
  }

  if (argc > fileNameIndex) {
    setLexInput(argv[fileNameIndex]);
  }

  int res = myParser.parse();

  return 0;
}
